@@ -56,7 +56,6 @@ Atom a_NET_DESKTOP_NAMES;
 Atom a_NET_ACTIVE_WINDOW;
 Atom a_NET_CLOSE_WINDOW;
 Atom a_NET_SUPPORTED;
-Atom a_NET_WM_DESKTOP;
 Atom a_NET_WM_STATE;
 Atom a_NET_WM_STATE_SKIP_TASKBAR;
 Atom a_NET_WM_STATE_SKIP_PAGER;
@@ -73,6 +72,7 @@ Atom a_NET_WM_WINDOW_TYPE_SPLASH;
 Atom a_NET_WM_WINDOW_TYPE_DIALOG;
 Atom a_NET_WM_WINDOW_TYPE_NORMAL;
 Atom a_NET_WM_DESKTOP;
+Atom a_NET_WM_PID;
 Atom a_NET_WM_NAME;
 Atom a_NET_WM_VISIBLE_NAME;
 Atom a_NET_WM_STRUT;
@@ -115,6 +115,7 @@ enum{
     I_NET_WM_WINDOW_TYPE_DIALOG,
     I_NET_WM_WINDOW_TYPE_NORMAL,
     I_NET_WM_DESKTOP,
+    I_NET_WM_PID,
     I_NET_WM_NAME,
     I_NET_WM_VISIBLE_NAME,
     I_NET_WM_STRUT,
@@ -158,8 +159,8 @@ pair height_pair[] = {
 };
 
 pair bool_pair[] = {
-    { 0, "false" },
-    { 1, "true" },
+    { 0, "0" },
+    { 1, "1" },
     { 0, NULL },
 };
 pair pos_pair[] = {
@@ -248,19 +249,19 @@ lxpanel_put_str( FILE* fp, const char* name, const char* val )
 {
     if( G_UNLIKELY( !val || !*val ) )
         return 0;
-    return lxpanel_put_line( fp, "%s = %s", name, val );
+    return lxpanel_put_line( fp, "%s=%s", name, val );
 }
 
 extern int
 lxpanel_put_bool( FILE* fp, const char* name, gboolean val )
 {
-    return lxpanel_put_line( fp, "%s = %s", name, val ? "true" : "false" );
+    return lxpanel_put_line( fp, "%s=%c", name, val ? '1' : '0' );
 }
 
 extern int
 lxpanel_put_int( FILE* fp, const char* name, int val )
 {
-    return lxpanel_put_line( fp, "%s = %d", name, val );
+    return lxpanel_put_line( fp, "%s=%d", name, val );
 }
 
 extern  int
@@ -302,18 +303,6 @@ lxpanel_get_line(char**fp, line *s)
     RET(s->type);
 }
 
-extern const char*
-lxpanel_get_file_manager( panel *p )
-{
-    return p->file_manager ? p->file_manager : "pcmanfm %";
-}
-
-extern const char*
-lxpanel_get_terminal( panel *p )
-{
-    return p->terminal ? p->terminal : "x-terminal-emulator";
-}
-
 int
 get_line_as_is(char** fp, line *s)
 {
@@ -386,6 +375,7 @@ void resolve_atoms()
     atom_names[ I_NET_WM_WINDOW_TYPE_DIALOG ] = "_NET_WM_WINDOW_TYPE_DIALOG";
     atom_names[ I_NET_WM_WINDOW_TYPE_NORMAL ] = "_NET_WM_WINDOW_TYPE_NORMAL";
     atom_names[ I_NET_WM_DESKTOP ] = "_NET_WM_DESKTOP";
+    atom_names[ I_NET_WM_PID ] = "_NET_WM_PID";
     atom_names[ I_NET_WM_NAME ] = "_NET_WM_NAME";
     atom_names[ I_NET_WM_VISIBLE_NAME ] = "_NET_WM_VISIBLE_NAME";
     atom_names[ I_NET_WM_STRUT ] = "_NET_WM_STRUT";
@@ -418,7 +408,6 @@ void resolve_atoms()
     a_NET_DESKTOP_NAMES = atoms[ I_NET_DESKTOP_NAMES ];
     a_NET_ACTIVE_WINDOW = atoms[ I_NET_ACTIVE_WINDOW ];
     a_NET_SUPPORTED = atoms[ I_NET_SUPPORTED ];
-    a_NET_WM_DESKTOP = atoms[ I_NET_WM_DESKTOP ];
     a_NET_WM_STATE = atoms[ I_NET_WM_STATE ];
     a_NET_WM_STATE_SKIP_TASKBAR = atoms[ I_NET_WM_STATE_SKIP_TASKBAR ];
     a_NET_WM_STATE_SKIP_PAGER = atoms[ I_NET_WM_STATE_SKIP_PAGER ];
@@ -682,8 +671,23 @@ get_net_wm_desktop(Window win)
     RET(desk);
 }
 
+GPid
+get_net_wm_pid(Window win)
+{
+    GPid pid = 0;
+    guint32 *data;
+
+    ENTER;
+    data = get_xaproperty (win, a_NET_WM_PID, XA_CARDINAL, 0);
+    if (data) {
+        pid = *data;
+        XFree (data);
+    }
+    RET(pid);
+}
+
 void
-get_net_wm_state(Window win, net_wm_state *nws)
+get_net_wm_state(Window win, NetWMState *nws)
 {
     Atom *state;
     int num3;
@@ -724,7 +728,7 @@ get_net_wm_state(Window win, net_wm_state *nws)
 
 
 void
-get_net_wm_window_type(Window win, net_wm_window_type *nwwt)
+get_net_wm_window_type(Window win, NetWMWindowType *nwwt)
 {
     Atom *state;
     int num3;
@@ -831,22 +835,22 @@ calculate_width(int scrw, int wtype, int allign, int margin,
 
 
 void
-calculate_position(panel *np)
+calculate_position(Panel *np)
 {
     int sswidth, ssheight, minx, miny;
 
     ENTER;
+    /* FIXME: Why this doesn't work? */
     if (0)  {
-        //if (np->curdesk < np->wa_len/4) {
+//        if (np->curdesk < np->wa_len/4) {
         minx = np->workarea[np->curdesk*4 + 0];
         miny = np->workarea[np->curdesk*4 + 1];
         sswidth  = np->workarea[np->curdesk*4 + 2];
         ssheight = np->workarea[np->curdesk*4 + 3];
     } else {
         minx = miny = 0;
-        sswidth  = gdk_screen_width();
-        ssheight = gdk_screen_height();
-
+        sswidth  = gdk_screen_get_width( gtk_widget_get_screen(np->topgwin) );
+        ssheight = gdk_screen_get_height( gtk_widget_get_screen(np->topgwin) );
     }
 
     if (np->edge == EDGE_TOP || np->edge == EDGE_BOTTOM) {
@@ -1160,6 +1164,45 @@ fb_button_new_from_file(gchar *fname, int width, int height, gulong hicolor, gbo
     RET(b);
 }
 
+GtkWidget *
+fb_button_new_from_file_with_colorlabel(gchar *fname, int width, int height,
+      gulong hicolor, gulong fcolor, gboolean keep_ratio, gchar *name)
+{
+    GtkWidget *b, *image, *box, *label;
+
+    ENTER;
+    b = gtk_event_box_new();
+    gtk_container_set_border_width(GTK_CONTAINER(b), 0);
+    GTK_WIDGET_UNSET_FLAGS (b, GTK_CAN_FOCUS);
+
+    box = gtk_hbox_new(FALSE, 0);
+    gtk_container_set_border_width(GTK_CONTAINER(box), 0);
+    GTK_WIDGET_UNSET_FLAGS (box, GTK_CAN_FOCUS);
+    gtk_container_add(GTK_CONTAINER(b), box);
+
+    image = gtk_image_new_from_file_scaled(fname, width, height, keep_ratio);
+    g_object_set_data(G_OBJECT(image), "hicolor", (gpointer)hicolor);
+    gtk_misc_set_padding (GTK_MISC(image), 0, 0);
+    if (hicolor > 0) {
+        gtk_widget_add_events(b, GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
+        g_signal_connect_swapped (G_OBJECT (b), "enter-notify-event",
+              G_CALLBACK (fb_button_enter), image);
+        g_signal_connect_swapped (G_OBJECT (b), "leave-notify-event",
+              G_CALLBACK (fb_button_leave), image);
+    }
+    gtk_box_pack_start(GTK_BOX(box), image, FALSE, FALSE, 0);
+    if (name) {
+        label =  gtk_label_new("");
+        char *lname = g_strdup_printf("<span color=\"#%06x\">%s</span>", fcolor, name);
+	gtk_label_set_markup(GTK_LABEL(label), lname);
+        gtk_misc_set_padding(GTK_MISC(label), 2, 0);
+	g_free(lname);
+        gtk_box_pack_end(GTK_BOX(box), label, FALSE, FALSE, 0);
+    }
+    gtk_widget_show_all(b);
+    RET(b);
+}
+
 GtkWidget *
 fb_button_new_from_file_with_label(gchar *fname, int width, int height,
       gulong hicolor, gboolean keep_ratio, gchar *name)
