@@ -36,9 +36,11 @@
 #define DEFAULT_CLOCK_FORMAT  "%R"
 
 typedef struct {
+    Panel* panel;
     GtkWidget *eb;
     GtkWidget *main;
     GtkWidget *clockw;
+    GtkWidget *calwin;
     GtkTooltips *tip;
     char *tfmt;
     char *cfmt;
@@ -46,12 +48,45 @@ typedef struct {
     short lastDay;
     int bold;
     int timer;
-    int usefontcolor;
-    GdkColor *gfontcolor;
+    gboolean cal_show;
 } dclock;
 
 static void
-update_label_orient( plugin* p );
+update_label_orient( Plugin* p );
+
+static GtkWidget *create_calendar()
+{
+    GtkWidget *win;
+    GtkWidget *calendar;
+
+    /* create a new window */
+    win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_default_size(GTK_WINDOW(win), 180, 180);
+    gtk_window_set_decorated(GTK_WINDOW(win), FALSE);
+    gtk_window_set_resizable (GTK_WINDOW(win), FALSE);
+    gtk_container_set_border_width(GTK_CONTAINER(win), 5);
+    gtk_window_set_skip_taskbar_hint(GTK_WINDOW(win), TRUE);
+    gtk_window_set_skip_pager_hint(GTK_WINDOW(win), TRUE);
+//    gtk_window_set_type_hint(GTK_WINDOW(win), GDK_WINDOW_TYPE_HINT_DOCK);
+    gtk_window_set_position(GTK_WINDOW(win), GTK_WIN_POS_MOUSE);
+    gtk_window_stick (GTK_WINDOW(win));
+
+    GtkVBox* box = gtk_vbox_new(FALSE, 0);
+
+    /* calendar */
+    calendar = gtk_calendar_new();
+    gtk_calendar_display_options(GTK_CALENDAR(calendar),
+                                 GTK_CALENDAR_SHOW_WEEK_NUMBERS |
+                                 GTK_CALENDAR_SHOW_DAY_NAMES |
+                                 GTK_CALENDAR_SHOW_HEADING);
+//    gtk_container_add(GTK_CONTAINER(win), calendar);
+    gtk_box_pack_start_defaults( box, calendar );
+    gtk_container_add(GTK_CONTAINER(win), box);
+
+    gtk_widget_show_all(win);
+
+    RET(win);
+}
 
 static int
 actionProcess( void *arg )
@@ -61,12 +96,29 @@ actionProcess( void *arg )
 }
 
 static  gboolean
-clicked( GtkWidget *widget, gpointer dummy, dclock *dc)
+clicked( GtkWidget *widget, GdkEventButton* evt, Plugin* plugin)
 {
+    dclock *dc = (dclock*)plugin->priv;
     ENTER2;
+    if( evt->button == 3 )  /* right button */
+    {
+        GtkMenu* popup = lxpanel_get_panel_menu( plugin->panel, plugin, FALSE );
+        gtk_menu_popup( popup, NULL, NULL, NULL, NULL, evt->button, evt->time );
+        return TRUE;
+    }
+
     if( dc->action ) {
         pthread_t actionThread;
         pthread_create(&actionThread, NULL, actionProcess, dc->action);
+    } else {
+        if (!dc->cal_show) {
+            dc->cal_show = TRUE;
+            dc->calwin = create_calendar();
+        } else {
+            dc->cal_show = FALSE;
+            gtk_widget_destroy(dc->calwin);
+            dc->calwin = NULL;
+        }
     }
     RET2(TRUE);
 }
@@ -89,12 +141,12 @@ clock_update(gpointer data )
     strftime(output, sizeof(output),
              (dc->cfmt ? dc->cfmt : DEFAULT_CLOCK_FORMAT), detail);
 
-    if (dc->bold&&dc->usefontcolor)
-        g_snprintf(str, 64, "<span color=\"#%06x\"><b>%s</b></span>", gcolor2rgb24(dc->gfontcolor), output);
+    if (dc->bold&& dc->panel->usefontcolor)
+        g_snprintf(str, 64, "<span color=\"#%06x\"><b>%s</b></span>", gcolor2rgb24( &dc->panel->gfontcolor ), output);
     else if (dc->bold)
         g_snprintf(str, 64, "<b>%s</b>", output);
-    else if (dc->usefontcolor)
-        g_snprintf(str, 64, "<span color=\"#%06x\">%s</span>", gcolor2rgb24(dc->gfontcolor), output);
+    else if ( dc->panel->usefontcolor)
+        g_snprintf(str, 64, "<span color=\"#%06x\">%s</span>", gcolor2rgb24(&dc->panel->gfontcolor), output);
     else
         g_snprintf(str, 64, "%s", output);
 
@@ -115,7 +167,7 @@ clock_update(gpointer data )
 
 
 static int
-dclock_constructor(plugin *p, char** fp)
+dclock_constructor(Plugin *p, char** fp)
 {
     line s;
     dclock *dc;
@@ -125,9 +177,12 @@ dclock_constructor(plugin *p, char** fp)
     g_return_val_if_fail(dc != NULL, 0);
     p->priv = dc;
 
+    dc->panel = p->panel;
+
     s.len = 256;
     dc->cfmt = dc->tfmt = dc->action = dc->bold = 0;
     dc->bold = TRUE;
+    dc->cal_show = FALSE;
     if( fp )
     {
         while (lxpanel_get_line(fp, &s) != LINE_BLOCK_END) {
@@ -156,9 +211,8 @@ dclock_constructor(plugin *p, char** fp)
     }
     dc->main = gtk_event_box_new();
 
-    if (dc->action)
-        g_signal_connect (G_OBJECT (dc->main), "button_press_event",
-              G_CALLBACK (clicked), (gpointer) dc);
+    g_signal_connect (G_OBJECT (dc->main), "button_press_event",
+          G_CALLBACK (clicked), (gpointer) p);
     dc->clockw = gtk_label_new("");
     gtk_misc_set_alignment(GTK_MISC(dc->clockw), 0.5, 0.5);
     gtk_misc_set_padding(GTK_MISC(dc->clockw), 4, 0);
@@ -167,8 +221,6 @@ dclock_constructor(plugin *p, char** fp)
     gtk_widget_show_all(dc->main);
 
     dc->tip = p->panel->tooltips;
-
-    dc->tip = gtk_tooltips_new();
 #if GLIB_CHECK_VERSION( 2, 10, 0 )
     g_object_ref_sink( dc->tip );
 #else
@@ -178,15 +230,9 @@ dclock_constructor(plugin *p, char** fp)
 
     dc->timer = g_timeout_add(1000, (GSourceFunc) clock_update, (gpointer)dc);
 
-    /* font color */
-    if (p->panel->usefontcolor)
-        dc->gfontcolor = &p->panel->gfontcolor;
-
-    dc->usefontcolor = p->panel->usefontcolor;
-
     clock_update( dc );
 
-    /* store the created plugin widget in plugin->pwid */
+    /* store the created Plugin widget in Plugin->pwid */
     p->pwid = dc->main;
 
     RET(1);
@@ -201,16 +247,17 @@ dclock_constructor(plugin *p, char** fp)
 
 
 static void
-dclock_destructor(plugin *p)
+dclock_destructor(Plugin *p)
 {
     dclock *dc = (dclock *)p->priv;
 
     ENTER;
-    dc = (dclock *) p->priv;
     if (dc->timer)
         g_source_remove(dc->timer);
 
     /* g_object_unref( dc->tip ); */
+	gtk_widget_destroy(dc->clockw);
+	gtk_widget_destroy(dc->main);
     g_free(dc->cfmt);
     g_free(dc->tfmt);
     g_free(dc->action);
@@ -218,7 +265,7 @@ dclock_destructor(plugin *p)
     RET();
 }
 
-static void apply_config( plugin* p )
+static void apply_config( Plugin* p )
 {
     /* NOTE: This dirty hack is used to force the update of tooltip
        because tooltip will be updated when dc->lastDay != today.
@@ -228,7 +275,7 @@ static void apply_config( plugin* p )
     clock_update( dc );
 }
 
-static void dclock_config( plugin *p, GtkWindow* parent )
+static void dclock_config( Plugin *p, GtkWindow* parent )
 {
     GtkWidget* dlg;
     dclock *dc = (dclock *)p->priv;
@@ -243,7 +290,7 @@ static void dclock_config( plugin *p, GtkWindow* parent )
     gtk_window_present( GTK_WINDOW(dlg) );
 }
 
-static void save_config( plugin* p, FILE* fp )
+static void save_config( Plugin* p, FILE* fp )
 {
     dclock *dc = (dclock *)p->priv;
     lxpanel_put_str( fp, "ClockFmt", dc->cfmt );
@@ -253,7 +300,7 @@ static void save_config( plugin* p, FILE* fp )
 }
 
 static void
-update_label_orient( plugin* p )
+update_label_orient( Plugin* p )
 {
     dclock *dc = (dclock *)p->priv;
     GtkLabel* label = GTK_LABEL(dc->clockw);
@@ -268,7 +315,7 @@ update_label_orient( plugin* p )
     gtk_label_set_angle( GTK_LABEL(label), angle );
 }
 
-plugin_class dclock_plugin_class = {
+PluginClass dclock_plugin_class = {
     fname: NULL,
     count: 0,
 
