@@ -32,7 +32,6 @@
 #include "misc.h"
 #include "plugin.h"
 
-//#define DEBUG
 #include "dbg.h"
 
 #include "glib-mem.h"
@@ -68,6 +67,7 @@ static const GtkTargetEntry target_table[] = {
 static const char desktop_ent[] = "Desktop Entry";
 
 typedef struct btn_t {
+    Plugin* plugin;
     GtkWidget* widget;
     gchar *desktop_id;
     gchar *image;
@@ -88,6 +88,8 @@ typedef struct launchbar {
     GtkWidget* config_dlg;
 } launchbar;
 
+void panel_config_save(Panel* panel);
+
 void btn_free( btn_t* btn )
 {
     g_free( btn->desktop_id );
@@ -98,32 +100,58 @@ void btn_free( btn_t* btn )
 }
 
 static gboolean
-on_button_pressed(GtkWidget *widget, GdkEventButton *event, btn_t *b )
+on_button_event(GtkWidget *widget, GdkEventButton *event, btn_t *b )
 {
     GtkWidget *image;
 
-    ENTER;
-    image = gtk_bin_get_child(GTK_BIN(widget));
-    g_assert(b != NULL);
-    if (event->type == GDK_BUTTON_RELEASE) {
-        if ((event->x >=0 && event->x < widget->allocation.width)
-              && (event->y >=0 && event->y < widget->allocation.height)) {
+    if( event->button == 1 )    /* left button */
+    {
+        image = gtk_bin_get_child(GTK_BIN(widget));
+        g_assert(b != NULL);
+        if (event->type == GDK_BUTTON_RELEASE) {
+            if ((event->x >=0 && event->x < widget->allocation.width)
+                  && (event->y >=0 && event->y < widget->allocation.height)) {
 
-            g_spawn_command_line_async(b->action, NULL);
+                g_spawn_command_line_async(b->action, NULL);
+            }
+            gtk_misc_set_padding (GTK_MISC(image), 0, 0);
+
+            //system(b->action);
+        } else if (event->type == GDK_BUTTON_PRESS) {
+
+            gtk_misc_set_padding (GTK_MISC(image), 0, 3);
+            //ERR("here\n");
         }
-        gtk_misc_set_padding (GTK_MISC(image), 0, 0);
-
-        //system(b->action);
-    } else if (event->type == GDK_BUTTON_PRESS) {
-
-        gtk_misc_set_padding (GTK_MISC(image), 0, 3);
-        //ERR("here\n");
+        return TRUE;
     }
-    RET(TRUE);
+    else if(event->button == 3) /* right click */
+    {
+        GtkMenu* popup = lxpanel_get_panel_menu( b->plugin->panel, b->plugin, TRUE );
+        GtkWidget* item;
+        char* title;
+
+#if 0
+        item = gtk_image_menu_item_new_with_label( _("Add Button") );
+        gtk_menu_shell_append( popup, item );
+        item = gtk_image_menu_item_new_with_label( _("Button Properties") );
+        gtk_menu_shell_append( popup, item );
+        /*
+        title = g_strdup_printf( _("Remove \"%s\""), b-> );
+        item = gtk_image_menu_item_new_with_label( _("Remove ") );
+        */
+        item = gtk_image_menu_item_new_with_label( _("Remove Button") );
+        gtk_menu_shell_append( popup, item );
+#endif
+        gtk_widget_show_all( (GtkWidget*)popup );
+
+        gtk_menu_popup( popup, NULL, NULL, NULL, NULL, event->button, event->time );
+        return TRUE;
+    }
+    return FALSE;
 }
 
 static void
-launchbar_destructor(plugin *p)
+launchbar_destructor(Plugin *p)
 {
     launchbar *lb = (launchbar *)p->priv;
 
@@ -187,7 +215,7 @@ drag_data_received_cb (GtkWidget        *widget,
 }
 
 static int
-read_button(plugin *p, char** fp)
+read_button(Plugin *p, char** fp)
 {
     launchbar *lb = (launchbar *)p->priv;
     gchar *fname;
@@ -199,6 +227,7 @@ read_button(plugin *p, char** fp)
     ENTER;
 
     btn = g_slice_new0( btn_t );
+    btn->plugin = p;
 
     s.len = 256;
     fname= NULL;
@@ -292,9 +321,9 @@ read_button(plugin *p, char** fp)
 
     //gtk_container_set_border_width(GTK_CONTAINER(button), 0);
     g_signal_connect ( button, "button-release-event",
-          G_CALLBACK (on_button_pressed), (gpointer) btn );
+          G_CALLBACK (on_button_event), (gpointer) btn );
     g_signal_connect ( button, "button-press-event",
-          G_CALLBACK (on_button_pressed), (gpointer) btn );
+          G_CALLBACK (on_button_event), (gpointer) btn );
 
     GTK_WIDGET_UNSET_FLAGS (button, GTK_CAN_FOCUS);
 
@@ -312,6 +341,7 @@ read_button(plugin *p, char** fp)
     lb->btns = g_slist_append( lb->btns, btn );
 
     gtk_widget_show(button);
+    plugin_widget_set_background( button, p->panel );
 
     g_free(fname);
 
@@ -328,7 +358,7 @@ read_button(plugin *p, char** fp)
 }
 
 static int
-launchbar_constructor(plugin *p, char **fp)
+launchbar_constructor(Plugin *p, char **fp)
 {
     launchbar *lb;
     line s;
@@ -352,7 +382,7 @@ launchbar_constructor(plugin *p, char **fp)
         "GtkButton::default-border = { 0, 0, 0, 0 }\n"
         "GtkButton::default-outside-border = { 0, 0, 0, 0 }\n"
         "}\n"
-        "widget '*' style 'launchbar-style'";
+        "widget '*launchbar*' style 'launchbar-style'";
 
     ENTER;
     gtk_rc_parse_string(launchbar_rc);
@@ -415,7 +445,7 @@ launchbar_constructor(plugin *p, char **fp)
 
 }
 
-static void save_config( plugin* p, FILE* fp )
+static void save_config( Plugin* p, FILE* fp )
 {
     launchbar *lb = (launchbar *)p->priv;
     GSList* l;
@@ -434,7 +464,7 @@ static void save_config( plugin* p, FILE* fp )
     }
 }
 
-static void orientation_changed( plugin* p )
+static void orientation_changed( Plugin* p )
 {
     launchbar *lb = (launchbar *)p->priv;
     GtkBox* newbox;
@@ -448,7 +478,7 @@ static void orientation_changed( plugin* p )
 }
 
 static void
-on_response( GtkDialog* dlg, int response, plugin* p )
+on_response( GtkDialog* dlg, int response, Plugin* p )
 {
     launchbar *lb = (launchbar *)p->priv;
     gtk_widget_destroy( GTK_WIDGET(dlg) );
@@ -461,7 +491,7 @@ static void on_add_btn_response( GtkDialog* dlg, int response, int* ret )
     gtk_main_quit();
 }
 
-static void on_add_btn( GtkButton* widget, plugin* p )
+static void on_add_btn( GtkButton* widget, Plugin* p )
 {
     launchbar *lb = (launchbar *)p->priv;
     GtkTreeView* view = (GtkTreeView*)g_object_get_data( (GObject *) lb->config_dlg, "view" );
@@ -534,7 +564,7 @@ static void on_add_btn( GtkButton* widget, plugin* p )
     gtk_widget_destroy( dlg );
 }
 
-static void on_remove_btn( GtkButton* widget, plugin* p )
+static void on_remove_btn( GtkButton* widget, Plugin* p )
 {
     launchbar *lb = (launchbar *)p->priv;
     GtkTreeView* view = (GtkTreeView*)g_object_get_data( lb->config_dlg, "view" );
@@ -555,7 +585,7 @@ static void on_remove_btn( GtkButton* widget, plugin* p )
     }
 }
 
-static void on_up_btn( GtkButton* widget, plugin* p )
+static void on_up_btn( GtkButton* widget, Plugin* p )
 {
     launchbar *lb = (launchbar *)p->priv;
     btn_t *btn;
@@ -584,7 +614,7 @@ static void on_up_btn( GtkButton* widget, plugin* p )
     gtk_tree_path_free( path );
 }
 
-static void on_down_btn( GtkButton* widget, plugin* p )
+static void on_down_btn( GtkButton* widget, Plugin* p )
 {
     launchbar *lb = (launchbar *)p->priv;
     btn_t *btn;
@@ -614,7 +644,7 @@ static void on_down_btn( GtkButton* widget, plugin* p )
     gtk_tree_path_free( path );
 }
 
-static void init_btn_list( plugin* p, GtkTreeView* view )
+static void init_btn_list( Plugin* p, GtkTreeView* view )
 {
     launchbar *lb = (launchbar *)p->priv;
     GtkListStore *list;
@@ -675,7 +705,7 @@ static void init_btn_list( plugin* p, GtkTreeView* view )
     g_object_set_data( lb->config_dlg, "view", view );
 }
 
-static void launchbar_config( plugin *p, GtkWindow* parent )
+static void launchbar_config( Plugin *p, GtkWindow* parent )
 {
     GtkWidget *dlg, *hbox, *vbox, *scroll, *view, *btn, *img;
     launchbar *lb = (launchbar *)p->priv;
@@ -735,11 +765,13 @@ static void launchbar_config( plugin *p, GtkWindow* parent )
         init_btn_list( p, view );
 
         gtk_widget_show_all( dlg );
+
+        g_object_weak_ref(dlg, panel_config_save, p->panel );
     }
     gtk_window_present( GTK_WINDOW(lb->config_dlg) );
 }
 
-plugin_class launchbar_plugin_class = {
+PluginClass launchbar_plugin_class = {
     fname: NULL,
     count: 0,
 
