@@ -30,7 +30,6 @@
 
 #include "ptk-app-menu.h"
 
-//#define DEBUG
 #include "dbg.h"
 
 /*
@@ -45,18 +44,20 @@
 typedef struct {
     GtkTooltips *tips;
     GtkWidget *menu, *box, *bg, *label;
+    char *fname, *caption;
     gulong handler_id;
     int iconsize, paneliconsize;
     GSList *files;
     gboolean has_system_menu;
     char* config_data;
     int sysmenu_pos;
+    char *config_start, *config_end;
 } menup;
 
 static guint idle_loader = 0;
 
 static void
-menu_destructor(plugin *p)
+menu_destructor(Plugin *p)
 {
     menup *m = (menup *)p->priv;
 
@@ -75,6 +76,8 @@ menu_destructor(plugin *p)
     /* The widget is destroyed in plugin_stop().
     gtk_widget_destroy(m->box);
     */
+    g_free(m->fname);
+    g_free(m->caption);
     g_free(m);
     RET();
 }
@@ -107,7 +110,7 @@ static void
 menu_pos(GtkMenu *menu, gint *x, gint *y, gboolean *push_in, GtkWidget *widget)
 {
     int ox, oy, w, h;
-    plugin *p;
+    Plugin *p;
 
     ENTER;
     p = g_object_get_data(G_OBJECT(widget), "plugin");
@@ -163,7 +166,7 @@ reload_system_menu( GtkMenu* menu )
     g_list_free( children );
 }
 
-static void show_menu( GtkWidget* widget, plugin* p, int btn, guint32 time )
+static void show_menu( GtkWidget* widget, Plugin* p, int btn, guint32 time )
 {
     menup* m = (menup*)p->priv;
     /* reload system menu items if needed */
@@ -172,7 +175,7 @@ static void show_menu( GtkWidget* widget, plugin* p, int btn, guint32 time )
         /* FIXME: Reload all system menus here.
                   This is dirty, but I don't know any better way. */
         for( l = p->panel->system_menus; l; l = l->next ) {
-            plugin* _p = (plugin*)l->data;
+            Plugin* _p = (Plugin*)l->data;
             menup* _m = (menup*)_p->priv;
             reload_system_menu( GTK_MENU(_m->menu) );
         }
@@ -184,29 +187,38 @@ static void show_menu( GtkWidget* widget, plugin* p, int btn, guint32 time )
 }
 
 static gboolean
-my_button_pressed(GtkWidget *widget, GdkEventButton *event, plugin* p)
+my_button_pressed(GtkWidget *widget, GdkEventButton *event, Plugin* plugin)
 {
     ENTER;
+
+    if( event->button == 3 )  /* right button */
+    {
+        GtkMenu* popup = lxpanel_get_panel_menu( plugin->panel, plugin, FALSE );
+        gtk_menu_popup( popup, NULL, NULL, NULL, NULL, event->button, event->time );
+        return TRUE;
+    }
+
     if ((event->type == GDK_BUTTON_PRESS)
           && (event->x >=0 && event->x < widget->allocation.width)
           && (event->y >=0 && event->y < widget->allocation.height)) {
-        show_menu( widget, p, event->button, event->time );
+        show_menu( widget, plugin, event->button, event->time );
     }
     RET(TRUE);
 }
 
 gboolean show_system_menu( gpointer system_menu )
 {
-    plugin* p = (plugin*)system_menu;
+    Plugin* p = (Plugin*)system_menu;
     menup* m = (menup*)p->priv;
     show_menu( m->bg, p, 0, GDK_CURRENT_TIME );
     return FALSE;
 }
 
 static GtkWidget *
-make_button(plugin *p, gchar *fname, gchar *name, GtkWidget *menu)
+make_button(Plugin *p, gchar *fname, gchar *name, GdkColor* tint, GtkWidget *menu)
 {
     int w, h;
+    char* title = NULL;
     menup *m;
 
     ENTER;
@@ -219,8 +231,39 @@ make_button(plugin *p, gchar *fname, gchar *name, GtkWidget *menu)
         w = p->panel->aw;
         h = 10000;
     }
-    m->bg = fb_button_new_from_file_with_label(fname, w, h, 0xFF0000, TRUE,
-          (p->panel->orientation == ORIENT_HORIZ ? name : NULL));
+
+    if( name )
+    {
+        /* load the name from *.directory file if needed */
+        if( g_str_has_suffix( name, ".directory" ) )
+        {
+            GKeyFile* kf = g_key_file_new();
+            char* dir_file = g_build_filename( "desktop-directories", name, NULL ); 
+            if( g_key_file_load_from_data_dirs( kf, dir_file, NULL, 0, NULL ) )
+            {
+                title = g_key_file_get_locale_string( kf, "Desktop Entry", "Name", NULL, NULL );
+            }
+            g_free( dir_file );
+            g_key_file_free( kf );
+        }
+        else
+            title = name;
+
+        /* FIXME: handle orientation problems */
+        if (p->panel->usefontcolor)
+            m->bg = fb_button_new_from_file_with_colorlabel(fname, w, h, gcolor2rgb24(tint),
+                p->panel->fontcolor, TRUE, title);
+        else
+            m->bg = fb_button_new_from_file_with_label(fname, w, h, gcolor2rgb24(tint), TRUE, title);
+
+        if( title != name )
+            g_free( title );
+    }
+    else
+    {
+        m->bg = fb_button_new_from_file(fname, w, h, gcolor2rgb24(tint), TRUE );
+    }
+
     gtk_widget_show(m->bg);
     gtk_box_pack_start(GTK_BOX(m->box), m->bg, FALSE, FALSE, 0);
 
@@ -233,13 +276,13 @@ make_button(plugin *p, gchar *fname, gchar *name, GtkWidget *menu)
 
 
 static GtkWidget *
-read_item(plugin *p, char** fp)
+read_item(Plugin *p, char** fp)
 {
     line s;
     gchar *name, *fname, *action;
     GtkWidget *item;
     menup *m = (menup *)p->priv;
-    command *cmd_entry = NULL;
+    Command *cmd_entry = NULL;
 
     ENTER;
     s.len = 256;
@@ -256,7 +299,7 @@ read_item(plugin *p, char** fp)
                 else if (!g_ascii_strcasecmp(s.t[0], "action"))
                     action = g_strdup(s.t[1]);
                 else if (!g_ascii_strcasecmp(s.t[0], "command")) {
-                    command *tmp;
+                    Command *tmp;
 
                     for (tmp = commands; tmp->name; tmp++) {
                         if (!g_ascii_strcasecmp(s.t[1], tmp->name)) {
@@ -304,7 +347,7 @@ read_item(plugin *p, char** fp)
 }
 
 static GtkWidget *
-read_separator(plugin *p, char **fp)
+read_separator(Plugin *p, char **fp)
 {
     line s;
 
@@ -320,13 +363,13 @@ read_separator(plugin *p, char **fp)
     RET(gtk_separator_menu_item_new());
 }
 
-static gboolean on_idle( panel* p )
+static gboolean on_idle( Panel* p )
 {
     GSList* l;
     /* Reload all system menus here.
         This is dirty, but I don't know any better way. */
     for( l = p->system_menus; l; l = l->next ) {
-        plugin* _p = (plugin*)l->data;
+        Plugin* _p = (Plugin*)l->data;
         menup* _m = (menup*)_p->priv;
         reload_system_menu( GTK_MENU(_m->menu) );
     }
@@ -335,7 +378,7 @@ static gboolean on_idle( panel* p )
 }
 
 static void
-read_system_menu(GtkMenu* menu, plugin *p, char** fp)
+read_system_menu(GtkMenu* menu, Plugin *p, char** fp)
 {
    line s;
    menup *m = (menup *)p->priv;
@@ -371,7 +414,7 @@ read_system_menu(GtkMenu* menu, plugin *p, char** fp)
 }
 
 static void
-read_include(plugin *p, char **fp)
+read_include(Plugin *p, char **fp)
 {
     ENTER;
 #if 0
@@ -408,14 +451,17 @@ read_include(plugin *p, char **fp)
 }
 
 static GtkWidget *
-read_submenu(plugin *p, char** fp, gboolean as_item)
+read_submenu(Plugin *p, char** fp, gboolean as_item)
 {
     line s;
     GtkWidget *mi, *menu;
     gchar name[256], *fname;
     menup *m = (menup *)p->priv;
+    GdkColor color={0, 0, 36 * 0xffff / 0xff, 96 * 0xffff / 0xff};
 
     ENTER;
+
+
     s.len = 256;
     menu = gtk_menu_new ();
     gtk_container_set_border_width(GTK_CONTAINER(menu), 0);
@@ -448,10 +494,14 @@ read_submenu(plugin *p, char** fp, gboolean as_item)
             gtk_widget_show(mi);
             gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
         } else if (s.type == LINE_VAR) {
+            m->config_start = *fp;
             if (!g_ascii_strcasecmp(s.t[0], "image"))
                 fname = expand_tilda(s.t[1]);
             else if (!g_ascii_strcasecmp(s.t[0], "name"))
                 strcpy(name, s.t[1]);
+	    /* FIXME: tintcolor will not be saved.  */
+            else if (!g_ascii_strcasecmp(s.t[0], "tintcolor"))
+                gdk_color_parse( s.t[1], &color);
             else {
                 ERR("menu: unknown var %s\n", s.t[0]);
                 goto error;
@@ -481,7 +531,9 @@ read_submenu(plugin *p, char** fp, gboolean as_item)
         gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), menu);
         RET(mi);
     } else {
-        mi = make_button(p, fname, name, menu);
+        m->fname = g_strdup(fname);
+        m->caption = g_strdup(name);
+        mi = make_button(p, fname, name, &color, menu);
         if (fname)
             g_free(fname);
         RET(mi);
@@ -496,11 +548,11 @@ read_submenu(plugin *p, char** fp, gboolean as_item)
 }
 
 static int
-menu_constructor(plugin *p, char **fp)
+menu_constructor(Plugin *p, char **fp)
 {
     menup *m;
     static char default_config[] =
-        "image=" PACKAGE_DATA_DIR "/lxpanel/images/my-computer.svg\n"
+        "image=" PACKAGE_DATA_DIR "/lxpanel/images/my-computer.png\n"
         "system {\n"
         "}\n"
         "separator {\n"
@@ -508,10 +560,6 @@ menu_constructor(plugin *p, char **fp)
         "item {\n"
             "command=run\n"
         "}\n"
-        "item {\n"
-            "image=" PACKAGE_DATA_DIR "/lxpanel/images/gnome-setting.svg\n"
-            "command = configure\n"
-        "}\n"
         "separator {\n"
         "}\n"
         "item {\n"
@@ -524,6 +572,8 @@ menu_constructor(plugin *p, char **fp)
     ENTER;
     m = g_new0(menup, 1);
     g_return_val_if_fail(m != NULL, 0);
+    m->fname = NULL;
+    m->caption = NULL;
     p->priv = m;
 
     //gtk_rc_parse_string(menu_rc);
@@ -541,20 +591,20 @@ menu_constructor(plugin *p, char **fp)
     if( ! fp )
         fp = &config_default;
 
-    config_start = *fp;
+    m->config_start = *fp;
     if (!read_submenu(p, fp, FALSE)) {
         ERR("menu: plugin init failed\n");
         goto error;
     }
-    config_end = *fp - 1;
-    while( *config_end != '}' && config_end > config_start ) {
-        --config_end;
+    m->config_end = *fp - 1;
+    while( *m->config_end != '}' && m->config_end > m->config_start ) {
+        --m->config_end;
     }
-    if( *config_end == '}' )
-        --config_end;
+    if( *m->config_end == '}' )
+        --m->config_end;
 
-    m->config_data = g_strndup( config_start,
-                                (config_end-config_start) );
+    m->config_data = g_strndup( m->config_start,
+                                (m->config_end-m->config_start) );
 
     p->pwid = m->box;
 
@@ -565,9 +615,11 @@ menu_constructor(plugin *p, char **fp)
     RET(0);
 }
 
-static void save_config( plugin* p, FILE* fp )
+static void save_config( Plugin* p, FILE* fp )
 {
     menup* menu = (menup*)p->priv;
+    lxpanel_put_str( fp, "name", menu->caption );
+    lxpanel_put_str( fp, "image", menu->fname );
     if( menu->config_data ) {
         char** lines = g_strsplit( menu->config_data, "\n", 0 );
         char** line;
@@ -580,7 +632,25 @@ static void save_config( plugin* p, FILE* fp )
     }
 }
 
-plugin_class menu_plugin_class = {
+static void apply_config(Plugin* p)
+{
+    /* FIXME: update menu for new setting */
+}
+
+static void menu_config( Plugin *p, GtkWindow* parent )
+{
+    GtkWidget* dlg;
+    menup* menu = (menup*)p->priv;
+    dlg = create_generic_config_dlg( _(p->class->name),
+                                     GTK_WIDGET(parent),
+                                    (GSourceFunc) apply_config, (gpointer) p,
+                                     _("Icon"), &menu->fname, G_TYPE_STRING,
+                                     _("Caption"), &menu->caption, G_TYPE_STRING,
+                                     NULL );
+    gtk_window_present( GTK_WINDOW(dlg) );
+}
+
+PluginClass menu_plugin_class = {
     fname: NULL,
     count: 0,
 
@@ -591,7 +661,7 @@ plugin_class menu_plugin_class = {
 
     constructor : menu_constructor,
     destructor  : menu_destructor,
-    config : NULL,
+    /* config : menu_config, */
     save : save_config
 };
 
